#include <ddraw.h>
#include <sys/timeb.h>
#include <stdio.h>
#include <stdlib.h>

#define RES_X 800
#define RES_Y 600
#define RES_BITDEPTH 16

LPDIRECTDRAW7 m_lpDD;
LPDIRECTDRAWSURFACE7	m_lpDDSFront;
LPDIRECTDRAWSURFACE7	m_lpDDSBack;


HRESULT InitDDraw( HWND hWnd )
{
	m_lpDD = NULL;
	m_lpDDSFront = m_lpDDSBack = NULL;

    HRESULT dwResult;

    dwResult = DirectDrawCreateEx(NULL, (void**)&m_lpDD, IID_IDirectDraw7, NULL);
    if(dwResult != DD_OK) {
//        TRACE("Cannot create direct draw object due %d\n", dwResult);
        return dwResult;
    }

    dwResult = m_lpDD->SetCooperativeLevel(hWnd, DDSCL_EXCLUSIVE|
                                                DDSCL_FULLSCREEN|
                                                DDSCL_ALLOWREBOOT);
    if(dwResult != DD_OK) {
//        TRACE("Cannot set cooperative level due %d\n", dwResult);
        return dwResult;
    }

    dwResult = m_lpDD->SetDisplayMode(RES_X,RES_Y, RES_BITDEPTH, 0, 0);
    if(dwResult != DD_OK) {
//        TRACE("Cannot set display mode due %d\n", dwResult);
        return dwResult;
    }

	DDSURFACEDESC2 ddsd;

	ZeroMemory(&ddsd, sizeof(ddsd));
	ddsd.dwSize = sizeof(ddsd);
	ddsd.dwFlags = DDSD_CAPS | DDSD_BACKBUFFERCOUNT;
	ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE | DDSCAPS_FLIP | 
			  DDSCAPS_COMPLEX;
	ddsd.dwBackBufferCount = 1;

	dwResult = m_lpDD->CreateSurface(&ddsd, &m_lpDDSFront, NULL);
	if(dwResult != DD_OK) {
//		TRACE("Cannot create front buffer due %d\n", dwResult);
		return dwResult;
	} 

	DDSCAPS2 ddscaps;

    ZeroMemory(&ddscaps, sizeof(ddscaps));
    ddscaps.dwCaps = DDSCAPS_BACKBUFFER;

	dwResult = m_lpDDSFront->GetAttachedSurface(&ddscaps, &m_lpDDSBack);
    if(dwResult != DD_OK) {
//		TRACE("Cannot create back buffer due %d\n", dwResult);
		return dwResult;
	}

	return dwResult;
}



VOID Cleanup()
{
	if(m_lpDD) {
		if(m_lpDDSBack)
			m_lpDDSBack->Release();
		if(m_lpDDSFront)
			m_lpDDSFront->Release();		
		m_lpDDSFront = m_lpDDSBack = NULL;
		m_lpDD->RestoreDisplayMode();
		m_lpDD->Release();
		m_lpDD = NULL;
	}
}

double	fps	= 1000;
int		i	= 0;

struct _timeb start, stop;

HPEN	hp = CreatePen(PS_SOLID, 1, RGB(155,255,255));
HBRUSH 	hb = CreateSolidBrush(RGB(0, 0, 0));
HDC hdc;
char sfps[20];

struct bod {
	double	x,
			y,
			vx,
			vy;
};

#define POC 100
bod body[POC];

RECT obd = {0, 0, RES_X, RES_Y};

void Render()
{
	if (m_lpDDSBack->GetDC(&hdc) == DD_OK)
	{
		FillRect(hdc, &obd, hb);


		sprintf(sfps, "FPS: %.1f", fps);
		SetBkColor( hdc, RGB( 0, 0, 0 ) );
		SetTextColor( hdc, RGB( 0, 0, 255 ) );
		TextOut( hdc, 0, 0, sfps, lstrlen(sfps) );

		SelectObject(hdc, hp);
		for(int j = 0; j < POC; j ++) {
			LineTo(hdc, (int)body[j].x, (int)body[j].y);
		}
		m_lpDDSBack->ReleaseDC(hdc);

		for(j = 0; j < POC; j ++) {
			body[j].x += body[j].vx;
			body[j].y += body[j].vy;
			if (body[j].x <= 0)	{
				body[j].vx *= -1;
				body[j].x = 0;
			}
			if (body[j].x >= RES_X)	{
				body[j].vx *= -1;
				body[j].x = RES_X;
			}
			if (body[j].y <= 0) {
				body[j].vy *= -1;
				body[j].y = 0;
			}
			if (body[j].y >= RES_Y) {
				body[j].vy *= -1;
				body[j].y = RES_Y;
			}
			body[j].vy += 0.01;
		}

		i ++;
		if (i > 50)
		{
			_ftime(&stop);
			fps = i/((stop.time - start.time) + 0.001*(stop.millitm - start.millitm));
			_ftime(&start);
			i = 0;
		}
	}
	m_lpDDSFront->Flip(NULL, DDFLIP_WAIT);
}


LRESULT WINAPI MsgProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
    switch( msg )
    {
        case WM_DESTROY:
            PostQuitMessage( 0 );
            return 0;

        case WM_PAINT:
            ValidateRect( hWnd, NULL );
            return 0;
    }

    return DefWindowProc( hWnd, msg, wParam, lParam );
}



INT WINAPI WinMain( HINSTANCE hInst, HINSTANCE, LPSTR, INT )
{
    WNDCLASSEX wc = { sizeof(WNDCLASSEX), CS_CLASSDC, MsgProc, 0L, 0L, 
                      GetModuleHandle(NULL), NULL, NULL, NULL, NULL,
                      "Direct Draw Test", NULL };
    RegisterClassEx( &wc );

    HWND hWnd = CreateWindow( "Direct Draw Test", "Direct Draw Test 1", 
                              WS_OVERLAPPEDWINDOW, 100, 100, 300, 300,
                              GetDesktopWindow(), NULL, wc.hInstance, NULL );

    if( SUCCEEDED( InitDDraw( hWnd ) ) )
    { 
        ShowWindow( hWnd, SW_SHOWDEFAULT );
        UpdateWindow( hWnd );

        MSG msg;
		BOOL gm = TRUE;

		_ftime(&start);
		for (int j = 0; j < POC; j ++) {
			body[j].x = rand()*0.01;
			body[j].y = rand()*0.01;
			body[j].vx = rand()*0.0001;
			body[j].vy = rand()*0.0001;
		}

		while ( gm ) {
			if( PeekMessage( &msg, NULL, 0, 0, PM_NOREMOVE ) )
			{
				if( gm = GetMessage( &msg, NULL, 0, 0 ) )
				{
					TranslateMessage( &msg );
					DispatchMessage( &msg );
				}
			}
			else Render();
		}
    }

    Cleanup();
    UnregisterClass( "Direct Draw Test", wc.hInstance );
    return 0;
}



